/**
 * Klasse Fahrscheinautomat.
 * 
 * @author R. Punkenburg 
 * @version 2.2.2006
 */
public class Fahrscheinautomat
{
    private int fahrscheinWert;
    private boolean fahrscheinGewaehlt;
    private int summe;
    private Muenzpruefer muenzpruefer;
    public Drucker drucker;
    private Muenzschacht muenzen5;
    private Muenzschacht muenzen10;
    private Muenzschacht muenzen20;
    private Muenzschacht muenzen50;
    private Muenzschacht muenzen100;
    private Muenzschacht muenzen200;
    private Geldkassette geldkassette;
    private boolean passendZahlen;
    private String wgText;
    private Muenzrueckgabe_GUI mRueckgabe;
    private Service_GUI service;

    /**
     * Konstruktor fr Objekte der Klasse Fahrscheinautomat;
     * Es ist kein Fahrschein gewhlt;
     * Es ist keine Mnze eingeworfen worden.
     */
    public Fahrscheinautomat()
    {
        fahrscheinWert = 0;
        fahrscheinGewaehlt = false;
        summe = 0;
        muenzpruefer = new Muenzpruefer();
        drucker = new Drucker();
        muenzen5 = new Muenzschacht("5");
        muenzen10 = new Muenzschacht("10");
        muenzen20 = new Muenzschacht("20");
        muenzen50 = new Muenzschacht("50");
        muenzen100 = new Muenzschacht("100");
        muenzen200 = new Muenzschacht("200");
        geldkassette = new Geldkassette();
        passendZahlen = passendZahlen();
        wgText = "";
        mRueckgabe = new Muenzrueckgabe_GUI();
        service = new Service_GUI(this);
        
    }
    /**
     * Den Preis des gewhlten Fahrscheins liefern
     * @return der Preis des gewhlten Fahrscheins
     */
    public int getFahrscheinWert()
    {
        return fahrscheinWert;
    }
    
    /**
     * Die Summe der eingeworfenen Mnzen liefern
     * @return die Summe der eingeworfenen Mnzen
     */
    public int getSumme()
    {  
        return summe;
    }
    
    /**
     * Den noch zu zahlenden Restbetrag liefern
     * @return den noch zu zahlenden Restbetrag
     */
    public int nochZuZahlen()
    {
        return fahrscheinWert-summe;
    }
    
    /**
     * Methode zum Verarbeiten der eingeworfenen Mnzen;
     * nur wenn ein Fahrschein gewhlt worden ist, werden Mnzeinwrfe akzeptiert;
     * wenn ausreichend Mnzen eingeworfen worden sind, wird der Vorgang abgearbeitet.
     * @param  wert  Wert der eingeworfenen Mnze
     */
    public boolean muenzEinwurf(int wert)
    {
        // Mnze prfen
        if (muenzpruefer.pruefe(wert) == 0)  // Mnze ist Falschgeld
        {
            return false;
        }
        
        else
        {
            if (passendZahlen && summe+wert>fahrscheinWert) 
            {
                verbucheMuenze(wert);
                wechselgeldAuszahlen(wert);
                return false;
            }
            else
            {
                verbucheMuenze(wert);
                summe += wert;
                return true;
            }
        }
    } // muenzEinwurf
    
    /**
     * Falls ausreichend Geld eingeworfen wurde, wird der Vorgang abgearbeitet und
     * true zurckgegeben, andernfalls geschieht nichts und false wird zurckgegeben
     * @retrun true, falls die Summe ausreichend war, sonst false
     */
    public boolean summeVerarbeiten()
    {
        if (summe< fahrscheinWert) return false;
        else
        {
            drucker.druckeFahrschein(fahrscheinWert);
            wechselgeldAuszahlen(summe-fahrscheinWert);
            // Attribute zurcksetzen
            fahrscheinGewaehlt = false;
            fahrscheinWert = 0;
            summe = 0;
            return true;
        }
    } // summe verarbeiten
    
    /**
     * Prfen, ob die Rckgabe von Wechselgeld eventuell unmglich ist
     * @return true, falls die Rckgabe von Wechselgeld eventuell unmglich ist, sonst false
     */
    public boolean passendZahlen()
    {
        if (   muenzen5.getAnzahl()<1
            || muenzen10.getAnzahl()<1
            || muenzen20.getAnzahl()<2
            || muenzen50.getAnzahl()<1
            || muenzen100.getAnzahl()<1) return true;
        else return false;
    }
    
    /**
     * Die Wechselgeldauszahlung verbuchen und einen Text zusammenbasteln, der die
     * Zusammensetzung des Wechselgeldes zeigt
     * @param der Betrag, der zurckgezahlt werden muss
     */
    private void wechselgeldAuszahlen(int betrag)
    {
        wgText = "";
        while (betrag > 0)
        {
            if (betrag/200 >0)
            {muenzen200.auswerfen();betrag -=200;wgText += "200 Cent - ";}
            
            else if (betrag/100 >0)
            {muenzen100.auswerfen();betrag -=100;wgText += "100 Cent - ";}
            
            else if (betrag/50 >0)
            {muenzen50.auswerfen();betrag -=50;wgText += "50 Cent - ";}
            
            else if (betrag/20 >0)
            {muenzen20.auswerfen();betrag -=20;wgText += "20 Cent - ";}
            
            else if (betrag/10 >0)
            {muenzen10.auswerfen();betrag -=10;wgText += "10 Cent - ";}
            
            else if (betrag/5 >0)
            {muenzen5.auswerfen();betrag -=5;wgText += "5 Cent - ";}
            
            else System.out.println("wechselgeldAuszahlen: Fehler!!!!");
            
        }
        mRueckgabe.zeigeWG(wgText);
    }
    
    
    /**
     * Methode zum Verbuchen der eingeworfenen Mnzen in dem entsprechenden
     * Mnzschacht bzw. in der Geldkassette;
     * wenn der Mnzschacht voll ist, wird die Mnze in der Geldkassette abgelegt, sonst im Mnzschacht.
     * @param  wert  Wert der eingeworfenen Mnze
     */
    private void verbucheMuenze(int wert)
    {
        if (wert==5) 
        {
            if (muenzen5.istVoll()) geldkassette.aufstocken(wert);
            else muenzen5.aufstocken();
        }
        if (wert==10) 
        {
            if (muenzen10.istVoll()) geldkassette.aufstocken(wert);
            else muenzen10.aufstocken();
        }
        if (wert==20) 
        {
            if (muenzen20.istVoll()) geldkassette.aufstocken(wert);
            else muenzen20.aufstocken();
        }
        if (wert==50) 
        {
            if (muenzen50.istVoll()) geldkassette.aufstocken(wert);
            else muenzen50.aufstocken();
        }
        if (wert==100) 
        {
            if (muenzen100.istVoll()) geldkassette.aufstocken(wert);
            else muenzen100.aufstocken();
        }
        if (wert==200) 
        {
            if (muenzen200.istVoll()) geldkassette.aufstocken(wert);
            else muenzen200.aufstocken();
        }
    } // verbucheMuenze
    
    
    
    /**
     * Methode zum Auswhlen des Fahrscheins;
     * Typ = 1 : Fahrschein fr 145 Cent;
     * Typ = 2 : Fahrschein fr 185 Cent;
     * Typ = 3 : Fahrschein fr 220 Cent;
     * alle anderen Eingaben sind ungltig
     * @param  typ  Typ des Fahrscheins
     */
    public void waehleFahrschein(int typ)
    {
        if (drucker.leer())
        {
            System.out.println("FSA: Der Automat ist auer Betrieb!");
            return; // Methode abbrechen
        }
        
        if (fahrscheinGewaehlt)
        {
            System.out.println("FSA: Sie haben bereits einen Fahrschein gewhlt!");
            System.out.println("FSA: Weitere Mnzen einwerfen oder ABBRUCH whlen!");
            return;
        }
        
        if (typ == 1)
        {
            fahrscheinGewaehlt = true;
            fahrscheinWert = 145;
            //System.out.println("FSA: Sie mchten einen Fahrschein fr 1,45 Euro. Bitte Mnzen einwerfen!");
        }
        else if (typ == 2)
        {
            fahrscheinGewaehlt = true;
            fahrscheinWert = 185;
            //System.out.println("FSA: Sie mchten einen Fahrschein fr 1,85 Euro. Bitte Mnzen einwerfen!");
        }
        else if (typ == 3)
        {
            fahrscheinGewaehlt = true;
            fahrscheinWert = 220;
            //System.out.println("FSA: Sie mchten einen Fahrschein fr 2,20 Euro. Bitte Mnzen einwerfen!");
        }
        else
        {
            fahrscheinGewaehlt = false;
            fahrscheinWert = 0;
            System.out.println("FSA: Ungltige Fahrscheinwahl!");
        }
        passendZahlen = passendZahlen(); // Zustand abfragen
        if (passendZahlen)
        {
            //System.out.println("FSA: Bitte passend zahlen!");
        }
    } // waehleFahrschein
    
    /**
     * Methode zum Abbrechen eines laufenden Vorgangs;
     * evtl. bereits eingeworfene Geld wird zurckgegeben;
     * kein Fahrschein ist gewhlt
     */
    public void abbrechen()
    {
        if (!fahrscheinGewaehlt) return; // nichts tun, weil nichts rckgngig zu machen ist
        //System.out.println("FSA: Vorgang abgebrochen!");
        if (summe >0) 
        {
            //System.out.println("FSA: Sie erhalten "+summe+" Cent zurck.");
            wechselgeldAuszahlen(summe);
        }
        fahrscheinGewaehlt = false;
        fahrscheinWert = 0;
        summe = 0;
    } // abbrechen
    
    /**
     * Druckeranzeige und Mnzrckgabeanzeige lschen
     */
    public void clear()
    {
        drucker.clear();
        mRueckgabe.clear();
    }
    
    /**
     * Den Text der Service-Anzeige aktualisieren
     */
    public void serviceAktualisieren()
    {
        service.zeige(0,"Leerformulare: "+drucker.getLeerFormulare());
        service.zeige(1,"Geldkassette: "+geldkassette.getInhalt());
        service.zeige(2,"5-Cent-Mnzen: "+muenzen5.getAnzahl());
        service.zeige(3,"10-Cent-Mnzen: "+muenzen10.getAnzahl());
        service.zeige(4,"20-Cent-Mnzen: "+muenzen20.getAnzahl());
        service.zeige(5,"50-Cent-Mnzen: "+muenzen50.getAnzahl());
        service.zeige(6,"100-Cent-Mnzen: "+muenzen100.getAnzahl());
        service.zeige(7,"200-Cent-Mnzen: "+muenzen200.getAnzahl());
    }
    
    /**
     * Prfen, ob der Automat betriebsbereit ist
     * @return true, falls der Automat betriebsbereit ist, sonst false
     */
    public boolean bereit()
    {
        return !drucker.leer();
    }
    
    
    
} // Fahrscheinautomat
